@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset: 4 -*-
  * libglade - a library for building interfaces from XML files at runtime
- * Copyright (C) 1998-2001  James Henstridge <james@daa.com.au>
+ * Copyright (C) 1998-2002  James Henstridge <james@daa.com.au>
  *
  * glade-xml.c: implementation of core public interface functions
  *
@@ -20,7 +20,7 @@
  * Boston, MA  02111-1307, USA.
  */
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include <config.h>
 #endif
 
 #include <stdlib.h>
@@ -117,13 +117,12 @@ glade_xml_init (GladeXML *self)
     priv->tree = NULL;
     priv->tooltips = gtk_tooltips_new();
     gtk_tooltips_enable(priv->tooltips);
-    gtk_object_ref(GTK_OBJECT(priv->tooltips));
+    g_object_ref(priv->tooltips);
     gtk_object_sink(GTK_OBJECT(priv->tooltips));
     priv->name_hash = g_hash_table_new(g_str_hash, g_str_equal);
     priv->signals = g_hash_table_new(g_str_hash, g_str_equal);
-    priv->radio_groups = g_hash_table_new (g_str_hash, g_str_equal);
     priv->toplevel = NULL;
-    priv->accel_groups = NULL;
+    priv->accel_group = NULL;
     priv->default_widget = NULL;
     priv->focus_widget = NULL;
     priv->deferred_props = NULL;
@@ -236,7 +235,7 @@ glade_xml_new_from_buffer(const char *buffer, int size, const char *root,
  */
 void
 glade_xml_signal_connect (GladeXML *self, const char *handlername,
-			  GtkSignalFunc func)
+			  GCallback func)
 {
     GList *signals;
 
@@ -249,24 +248,21 @@ glade_xml_signal_connect (GladeXML *self, const char *handlername,
 	GladeSignalData *data = signals->data;
 
 	if (data->connect_object) {
-	    GtkObject *other = g_hash_table_lookup(self->priv->name_hash,
+	    GObject *other = g_hash_table_lookup(self->priv->name_hash,
 						   data->connect_object);
-	    if (data->signal_after)
-		gtk_signal_connect_object_after(data->signal_object,
-						data->signal_name,
-						func, other);
-	    else
-		gtk_signal_connect_object(data->signal_object,
-					  data->signal_name, func, other);
+
+	    g_signal_connect_object(data->signal_object, data->signal_name,
+			func, other, (data->signal_after ? G_CONNECT_AFTER : 0)
+					| G_CONNECT_SWAPPED);
 	} else {
 	    /* the signal_data argument is just a string, but may be
 	     * helpful for someone */
 	    if (data->signal_after)
-		gtk_signal_connect_after(data->signal_object,
-					 data->signal_name, func, NULL);
+		g_signal_connect_after(data->signal_object, data->signal_name,
+				       func, NULL);
 	    else
-		gtk_signal_connect(data->signal_object, data->signal_name,
-				   func, NULL);
+		g_signal_connect(data->signal_object, data->signal_name,
+				 func, NULL);
 	}
     }
 }
@@ -275,7 +271,7 @@ static void
 autoconnect_foreach(const char *signal_handler, GList *signals,
 		    GModule *allsymbols)
 {
-    GtkSignalFunc func;
+    GCallback func;
 
     if (!g_module_symbol(allsymbols, signal_handler, (gpointer *)&func))
 	g_warning("could not find signal handler '%s'.", signal_handler);
@@ -285,24 +281,21 @@ autoconnect_foreach(const char *signal_handler, GList *signals,
 	    if (data->connect_object) {
 		GladeXML *self = glade_get_widget_tree(
 					GTK_WIDGET(data->signal_object));
-		GtkObject *other = g_hash_table_lookup(self->priv->name_hash,
+		GObject *other = g_hash_table_lookup(self->priv->name_hash,
 						       data->connect_object);
-		if (data->signal_after)
-		    gtk_signal_connect_object_after(data->signal_object,
-						    data->signal_name,
-						    func, other);
-		else
-		    gtk_signal_connect_object(data->signal_object,
-					      data->signal_name, func, other);
+
+		g_signal_connect_object(data->signal_object, data->signal_name,
+			func, other, (data->signal_after ? G_CONNECT_AFTER : 0)
+					| G_CONNECT_SWAPPED);
 	    } else {
 		/* the signal_data argument is just a string, but may
 		 * be helpful for someone */
 		if (data->signal_after)
-		    gtk_signal_connect_after(data->signal_object,
-					     data->signal_name, func, NULL);
+		    g_signal_connect_after(data->signal_object,
+					   data->signal_name, func, NULL);
 		else
-		    gtk_signal_connect(data->signal_object, data->signal_name,
-				       func, NULL);
+		    g_signal_connect(data->signal_object, data->signal_name,
+				     func, NULL);
 	    }
 	}
 }
@@ -349,7 +342,7 @@ autoconnect_full_foreach(const char *signal_handler, GList *signals,
 
     for (; signals != NULL; signals = signals->next) {
 	GladeSignalData *data = signals->data;
-	GtkObject *connect_object = NULL;
+	GObject *connect_object = NULL;
 		
 	if (data->connect_object) {
 	    if (!self)
@@ -457,38 +450,34 @@ glade_xml_signal_autoconnect_full (GladeXML *self, GladeXMLConnectFunc func,
  * demonstration of how to use glade_xml_signal_connect_full.
  */
 typedef struct {
-    GtkSignalFunc func;
+    GCallback func;
     gpointer user_data;
 } connect_data_data;
 
 static void
-connect_data_connect_func(const gchar *handler_name, GtkObject *object,
+connect_data_connect_func(const gchar *handler_name, GObject *object,
 			  const gchar *signal_name, const gchar *signal_data,
-			  GtkObject *connect_object, gboolean after,
+			  GObject *connect_object, gboolean after,
 			  gpointer user_data)
 {
     connect_data_data *data = (connect_data_data *)user_data;
 
     if (connect_object) {
-	if (after)
-	    gtk_signal_connect_object_after(object, signal_name,
-					    data->func, connect_object);
-	else
-	    gtk_signal_connect_object(object, signal_name,
-				      data->func, connect_object);
+	g_signal_connect_object(object, signal_name, data->func,
+	    connect_object, (after ? G_CONNECT_AFTER : 0) | G_CONNECT_SWAPPED);
     } else {
 	if (after)
-	    gtk_signal_connect_after(object, signal_name,
-				     data->func, data->user_data);
+	    g_signal_connect_after(object, signal_name,
+				   data->func, data->user_data);
 	else
-	    gtk_signal_connect(object, signal_name,
-			       data->func, data->user_data);
+	    g_signal_connect(object, signal_name,
+			     data->func, data->user_data);
     }
 }
 
 void
 glade_xml_signal_connect_data (GladeXML *self, const char *handlername,
-			       GtkSignalFunc func, gpointer user_data)
+			       GCallback func, gpointer user_data)
 {
     connect_data_data data;
 
@@ -645,20 +634,22 @@ glade_xml_set_toplevel(GladeXML *xml, GtkWindow *window)
     xml->priv->default_widget = NULL;
     xml->priv->toplevel = window;
     /* new toplevel needs new accel group */
-    if (xml->priv->accel_groups)
-	glade_xml_pop_accel(xml);
-    /* maybe put an assert that xml->priv->accel_groups == NULL here? */
-    xml->priv->accel_groups = NULL;
-    /* the window should hold a reference to the tooltips object */
-    gtk_object_ref(GTK_OBJECT(xml->priv->tooltips));
-    g_object_set_qdata_full(G_OBJECT(window), glade_xml_tooltips_id,
-			    xml->priv->tooltips,
-			    (GDestroyNotify)gtk_object_unref);
+    if (xml->priv->accel_group)
+	g_object_unref(xml->priv->accel_group);
+    xml->priv->accel_group = NULL;
+
+    if (GTK_IS_WINDOW (window)) {
+	/* the window should hold a reference to the tooltips object */
+	g_object_ref(xml->priv->tooltips);
+	g_object_set_qdata_full(G_OBJECT(window), glade_xml_tooltips_id,
+				xml->priv->tooltips,
+				(GDestroyNotify)g_object_unref);
+    }
 }
 
 /**
  * glade_xml_handle_widget_prop
- * @xml: the GladeXML object
+ * @self: the GladeXML object
  * @widget: the property the widget to set the property on.
  * @prop_name: the name of the property.
  * @value_name: the name of the widget used as the value for the property.
@@ -697,46 +688,6 @@ glade_xml_handle_widget_prop(GladeXML *self, GtkWidget *widget,
     }
 }
 
-/**
- * glade_xml_push_accel:
- * @xml: the GladeXML object.
- *
- * Make a new accel group amd push onto the stack.  This is intended for use
- * by GtkNotebook so that each notebook page can set up its own accelerators.
- * Returns: The current GtkAccelGroup after push.
- */
-GtkAccelGroup *
-glade_xml_push_accel(GladeXML *xml)
-{
-    GtkAccelGroup *accel = gtk_accel_group_new();
-       
-    xml->priv->accel_groups = g_slist_prepend(xml->priv->accel_groups, accel);
-    return accel;
-}
-
-/**
- * glade_xml_pop_accel:
- * @xml: the GladeXML object.
- *
- * Pops the accel group.  This will usually be called after a GtkNotebook page
- * has been built.
- * Returns: The current GtkAccelGroup after pop.
- */
-GtkAccelGroup *
-glade_xml_pop_accel(GladeXML *xml)
-{
-    GtkAccelGroup *accel;
-
-    g_return_val_if_fail(xml->priv->accel_groups != NULL, NULL);
-
-    /* the accel group at the top of the stack */
-    accel = xml->priv->accel_groups->data;
-    xml->priv->accel_groups = g_slist_remove(xml->priv->accel_groups, accel);
-    /* unref the accel group that was at the top of the stack */
-    gtk_accel_group_unref(accel);
-    return xml->priv->accel_groups ? xml->priv->accel_groups->data : NULL;
-}
-
 /**
  * glade_xml_ensure_accel:
  * @xml: the GladeXML object.
@@ -749,13 +700,13 @@ glade_xml_pop_accel(GladeXML *xml)
 GtkAccelGroup *
 glade_xml_ensure_accel(GladeXML *xml)
 {
-    if (!xml->priv->accel_groups) {
-	glade_xml_push_accel(xml);
+    if (!xml->priv->accel_group) {
+	xml->priv->accel_group = gtk_accel_group_new();
 	if (xml->priv->toplevel)
 	    gtk_window_add_accel_group(xml->priv->toplevel,
-				       xml->priv->accel_groups->data);
+				       xml->priv->accel_group);
     }
-    return (GtkAccelGroup *)xml->priv->accel_groups->data;
+    return xml->priv->accel_group;
 }
 
 /* this is a private function */
@@ -769,7 +720,7 @@ glade_xml_add_signals(GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info)
 	GladeSignalData *data = g_new0(GladeSignalData, 1);
 	GList *list;
 
-	data->signal_object = GTK_OBJECT(w);
+	data->signal_object = G_OBJECT(w);
 	data->signal_name = sig->name;
 	data->connect_object = sig->object;
 	data->signal_after = sig->after;
@@ -887,9 +838,9 @@ glade_xml_add_accels(GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info)
     for (i = 0; i < info->n_accels; i++) {
 	GladeAccelInfo *accel = &info->accels[i];
 
-	debug(g_message("New Accel: key=%d,mod=%d -> %s:%s",
-			accel->key, accel->modifiers,
-			gtk_widget_get_name(w), accel->signal));
+	GLADE_NOTE(BUILD, g_message("New Accel: key=%d,mod=%d -> %s:%s",
+				    accel->key, accel->modifiers,
+				    gtk_widget_get_name(w), accel->signal));
 	gtk_widget_add_accelerator(w, accel->signal,
 				   glade_xml_ensure_accel(xml),
 				   accel->key, accel->modifiers,
@@ -918,9 +869,9 @@ glade_xml_add_accessibility_info(GladeXML *xml, GtkWidget *w, GladeWidgetInfo *i
 	    g_value_unset (&value);
         }
 	
-	debug(g_message("Adding accessibility property %s:%s",
-			info->atk_props[i].name,
-			info->atk_props[i].value));
+	GLADE_NOTE(BUILD, g_message("Adding accessibility property %s:%s",
+				    info->atk_props[i].name,
+				    info->atk_props[i].value));
     }
 
     /* Hang on, we're not done yet ;-) */
@@ -946,12 +897,6 @@ glade_xml_destroy_signals(char *key, GList *signal_datas)
     g_list_free(signal_datas);
 }
 
-static void
-free_radio_groups(gpointer key, gpointer value, gpointer user_data)
-{
-    g_free(key);  /* the string name */
-}
-
 static void
 remove_data_func(gpointer key, gpointer value, gpointer user_data)
 {
@@ -981,17 +926,11 @@ glade_xml_finalize(GObject *object)
 			     (GHFunc)glade_xml_destroy_signals, NULL);
 	g_hash_table_destroy(priv->signals);
 
-	/* the group name strings are owned by the GladeWidgetTree */
-	g_hash_table_foreach(self->priv->radio_groups,
-			     free_radio_groups, NULL);
-	g_hash_table_destroy(priv->radio_groups);
-
 	if (priv->tooltips)
-	    gtk_object_unref(GTK_OBJECT(priv->tooltips));
+	    g_object_unref(priv->tooltips);
 
-	/* there should only be at most one accel group on stack */
-	if (priv->accel_groups)
-	    glade_xml_pop_accel(self);
+	if (priv->accel_group)
+	    g_object_unref(priv->accel_group);
 
 	if (priv->tree)
 	    glade_interface_destroy(priv->tree);
@@ -1004,14 +943,18 @@ glade_xml_finalize(GObject *object)
 }
 
 /**
- * glade_set_custom_handler
- * @handler: the custom widget handler
- * @user_data: user data passed to the custom handler
+ * GladeXMLCustomWidgetHandler:
+ * @xml: the GladeXML object.
+ * @func_name: the function name.
+ * @name: the name of the widget to be created.
+ * @string1: the string1 property.
+ * @string2: the string2 property.
+ * @int1: the int1 property.
+ * @int2: the int2 property.
+ * @user_data: the data passed to glade_set_custom_handler()
  *
- * Calling this function allows you to override the default behaviour
- * when a Custom widget is found in an interface.  This could be used by
- * a language binding to call some other function, or to limit what
- * functions can be called to create custom widgets.
+ * This prototype is for a function that creates custom widgets.
+ * Returns: the GtkWidget.
  */
 static GtkWidget *
 default_custom_handler(GladeXML *xml, gchar *func_name, gchar *name,
@@ -1038,6 +981,16 @@ default_custom_handler(GladeXML *xml, gchar *func_name, gchar *name,
 static GladeXMLCustomWidgetHandler custom_handler = default_custom_handler;
 static gpointer custom_user_data = NULL;
 
+/**
+ * glade_set_custom_handler:
+ * @handler: the custom widget handler
+ * @user_data: user data passed to the custom handler
+ *
+ * Calling this function allows you to override the default behaviour
+ * when a Custom widget is found in an interface.  This could be used by
+ * a language binding to call some other function, or to limit what
+ * functions can be called to create custom widgets.
+ */
 void
 glade_set_custom_handler(GladeXMLCustomWidgetHandler handler,
 			 gpointer user_data)
@@ -1046,20 +999,7 @@ glade_set_custom_handler(GladeXMLCustomWidgetHandler handler,
     custom_user_data = user_data;
 }
 
-/**
- * glade_set_custom_handler
- * @xml: the GladeXML object
- * @func_name: the name of the widget creation function
- * @name: the name of the widget
- * @string1: the first string argument
- * @string2: the second string argument
- * @int1: the first integer argument
- * @int2: the second integer argument
- *
- * Invokes the custom widget creation function.
- * Returns: the new widget or NULL on error.
- */
-GtkWidget *
+static GtkWidget *
 glade_create_custom(GladeXML *xml, gchar *func_name, gchar *name,
 		    gchar *string1, gchar *string2, gint int1, gint int2)
 {
@@ -1067,6 +1007,37 @@ glade_create_custom(GladeXML *xml, gchar *func_name, gchar *name,
 				  int1, int2, custom_user_data);
 }
 
+static GtkWidget *
+custom_new (GladeXML *xml, GladeWidgetInfo *info)
+{
+    GtkWidget *wid = NULL;
+    gchar *func_name = NULL, *string1 = NULL, *string2 = NULL;
+    gint int1 = 0, int2 = 0;
+    int i;
+    
+    for (i = 0; i < info->n_properties; i++) {
+	char *name  = info->properties[i].name;
+	char *value = info->properties[i].value;
+	
+	if (!strcmp(name, "creation_function"))
+	    func_name = value;
+	else if (!strcmp(name, "string1"))
+	    string1 = value;
+	else if (!strcmp(name, "string2"))
+	    string2 = value;
+	else if (!strcmp(name, "int1"))
+	    int1 = strtol(value, NULL, 0);
+	else if (!strcmp(name, "int2"))
+	    int2 = strtol(value, NULL, 0);
+    }
+    wid = glade_create_custom(xml, func_name, info->name, string1,
+			      string2, int1, int2);
+    /* fallback to prevent segfault */
+    if (wid == NULL)
+	wid = gtk_label_new("[custom widget creation failed]");
+    return wid;
+}
+
 /**
  * glade_enum_from_string
  * @type: the GType for this enum type.
@@ -1157,8 +1128,7 @@ glade_flags_from_string (GType type, const char *string)
 
 	    if (eos)
 		break;
-	} else
-	    i++;
+	}
     }
     
     g_free (flagstr);
@@ -1184,21 +1154,11 @@ glade_xml_build_interface(GladeXML *self, GladeInterface *iface,
 	wid = g_hash_table_lookup(iface->names, root);
 	g_return_if_fail(wid != NULL);
 	w = glade_xml_build_widget(self, wid);
-	if (GTK_IS_WINDOW(w)) {
-	    if (self->priv->focus_widget)
-		gtk_widget_grab_focus(self->priv->focus_widget);
-	    if (self->priv->default_widget)
-		gtk_widget_grab_default(self->priv->default_widget);
-	}
     } else {
 	/* build all toplevel nodes */
 	for (i = 0; i < iface->n_toplevels; i++) {
 	    w = glade_xml_build_widget(self, iface->toplevels[i]);
 	}
-	if (self->priv->focus_widget)
-	    gtk_widget_grab_focus(self->priv->focus_widget);
-	if (self->priv->default_widget)
-	    gtk_widget_grab_default(self->priv->default_widget);
     }
 }
 
@@ -1317,6 +1277,27 @@ invalidate_custom_prop_cache(GType type)
     g_free(children);
 }
 
+/**
+ * glade_register_custom_prop:
+ * @type: the GType of the widget.
+ * @prop_name: the name for the custom widget.
+ * @apply_prop: the function to be called when the custom property is found.
+ *
+ * Some properties are not (yet) handled through the GObject property
+ * code, so can not be handled by the generic code.  This function
+ * provides a way to register handlers for these properties.  Such
+ * handlers will apply for the GType @type and all its descendants.
+ */
+/**
+ * GladeApplyCustomPropFunc:
+ * @xml: the GladeXML object.
+ * @widget: the widget to apply the property to.
+ * @propname: the name of the property.
+ * @value: the value of the property.
+ *
+ * This prototype is used for custom property handlers registered with
+ * glade_register_custom_prop().
+ */
 void
 glade_register_custom_prop(GType type, const gchar *prop_name,
 			   GladeApplyCustomPropFunc apply_prop)
@@ -1440,7 +1421,10 @@ glade_xml_set_value_from_string (GladeXML *xml,
 	g_value_set_uchar(value, (guchar)string[0]);
 	break;
     case G_TYPE_BOOLEAN:
-	g_value_set_boolean(value, string[0] == 'T' || string[0] == 'y');
+	g_value_set_boolean(value, 
+			    g_ascii_tolower (string[0]) == 't'
+			    || g_ascii_tolower (string[0]) == 'y'
+			    || strtol (string, NULL, 0));
 	break;
     case G_TYPE_INT:
 	g_value_set_int(value, strtol(string, NULL, 0));
@@ -1670,7 +1654,7 @@ glade_standard_build_widget(GladeXML *xml, GType widget_type,
  * This sets the packing property on container @parent of widget
  * @child with @name to @value
  **/
-static void
+void
 glade_xml_set_packing_property (GladeXML   *self,
 				GtkWidget  *parent, GtkWidget  *child,
 				const char *name,   const char *value)
@@ -1734,7 +1718,7 @@ glade_standard_build_children(GladeXML *self, GtkWidget *parent,
 		self, parent, child,
 		info->children[i].properties[j].name,
 		info->children[i].properties[j].value);
-
+	
 	gtk_widget_thaw_child_notify(child);
 	g_object_unref(G_OBJECT(child));
     }
@@ -1814,27 +1798,31 @@ glade_xml_build_widget(GladeXML *self, GladeWidgetInfo *info)
 {
     GType type;
     GtkWidget *ret;
-
-    debug(g_message("Widget class: %s\tname: %s", info->class, info->name));
-    if (!strcmp(info->class, "Placeholder")) {
-	static int warned = 0;
-	if (!warned++)
-	    g_warning("placeholders exist in interface description");
-	ret = gtk_label_new("[placeholder]");
-	gtk_widget_show(ret);
-	return ret;
-    }
-    type = g_type_from_name(info->class);
-    if (type == G_TYPE_INVALID) {
-	char buf[50];
-	g_warning("unknown widget class '%s'", info->class);
-	g_snprintf(buf, 49, "[a %s]", info->class);
-	ret = gtk_label_new(buf);
-	gtk_widget_show(ret);
+    
+    GLADE_NOTE(BUILD, g_message("Widget class: %s\tname: %s",
+				info->class, info->name));
+    if (!strcmp (info->class, "Custom")) {
+	ret = custom_new (self, info);
     } else {
-	ret = get_build_data(type)->new(self, type, info);
+	type = g_type_from_name(info->class);
+	if (type == G_TYPE_INVALID) {
+	    char buf[50];
+	    g_warning("unknown widget class '%s'", info->class);
+	    g_snprintf(buf, 49, "[a %s]", info->class);
+	    ret = gtk_label_new(buf);
+	} else {
+	    ret = get_build_data(type)->new(self, type, info);
+	}
     }
+
+    if (GTK_IS_WINDOW (ret))
+	glade_xml_set_toplevel (self, GTK_WINDOW (ret));
+
     glade_xml_set_common_params(self, ret, info);
+
+    if (GTK_IS_WINDOW (ret))
+	glade_xml_set_toplevel (self, NULL);
+
     return ret;
 }
 
@@ -1922,6 +1910,7 @@ glade_xml_set_common_params(GladeXML *self, GtkWidget *widget,
 {
     GList *tmp;
     const GladeWidgetBuildData *data;
+    static GQuark visible_id = 0;
 
     /* get the build data */
     data = get_build_data(G_OBJECT_TYPE(widget));
@@ -1931,15 +1920,6 @@ glade_xml_set_common_params(GladeXML *self, GtkWidget *widget,
     gtk_widget_set_name(widget, info->name);
     glade_xml_add_accessibility_info(self, widget, info);
 
-#if 0
-    if (info->tooltip) {
-	gtk_tooltips_set_tip(self->priv->tooltips,
-			     widget,
-			     glade_xml_gettext(self, info->tooltip),
-			     NULL);
-    }
-#endif
-
     /* store this information as data of the widget. */
     g_object_set_qdata(G_OBJECT(widget), glade_xml_tree_id, self);
     g_object_set_qdata(G_OBJECT(widget), glade_xml_name_id, info->name);
@@ -1988,24 +1968,17 @@ glade_xml_set_common_params(GladeXML *self, GtkWidget *widget,
 	}
     }
 
-    if (data && data->build_children && info->children)
-	data->build_children(self, widget, info);
-}
-
-gchar **
-glade_xml_get_toplevel_names (GladeXML *self)
-{
-    int i;
-    gchar **ret;
-    GladeInterface *tree;
-    
-    tree = self->priv->tree;
-
-    ret = g_new (gchar *, tree->n_toplevels + 1);
-
-    for (i = 0; i < tree->n_toplevels; i++)
-	ret [i] = g_strdup (tree->toplevels [i]->name);
-    ret[i] = NULL;
-
-    return ret;
+    if (data && data->build_children && info->children) {
+	if (GTK_IS_CONTAINER (widget))
+	    data->build_children(self, widget, info);
+	else
+	    g_warning ("widget %s (%s) has children, but is not a GtkContainer.",
+		       info->name, g_type_name (G_TYPE_FROM_INSTANCE (widget)));
+    }
+
+    if (visible_id == 0)
+	visible_id = g_quark_from_static_string("Libglade::visible");
+
+    if (g_object_get_qdata(G_OBJECT(widget), visible_id))
+	gtk_widget_show(widget);
 }
